'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.listReducer = listReducer;
exports.createListReducers = createListReducers;

var _redux = require('redux');

var _constants = require('../constants');

var SET_OPTIONS = _constants.LIST.SET_OPTIONS,
    PAGE = _constants.LIST.PAGE,
    NEXT = _constants.LIST.NEXT,
    PREV = _constants.LIST.PREV,
    LAST = _constants.LIST.LAST,
    FIRST = _constants.LIST.FIRST,
    SET_PARAMS = _constants.LIST.SET_PARAMS;

var DEFAULT = {
  offset: 0,
  pageSize: 10,
  total: null,
  result: [],
  headers: {},
  params: {}
};

function listReducer(model, list) {
  var defaultState = _extends({}, DEFAULT, list.options);
  var reducer = function reducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultState;
    var _ref = arguments[1];
    var payload = _ref.payload,
        type = _ref.type;

    //REJECT actions without payloads & modelName
    if (!payload || !payload.modelName) {
      return state;
    }

    if (payload.modelName !== model.modelName || payload.listName !== list.name) {
      if (payload.modelName == model.modelName && type == _constants.CLEAR) {
        return _extends({}, defaultState);
      }
      return state;
    }

    console.log('reducing for ', list.name, payload.listName, payload.modelName, model.modelName);
    switch (type) {
      case SET_OPTIONS:
        return _extends({}, state, payload);
      case SET_PARAMS:
        return _extends({}, state, { params: _extends({}, state.params, payload) });
      case PAGE:
        return _extends({}, state, { offset: state.pageSize * payload.page });
      case _constants.RESPONSE.FIND:
        return _extends({}, state, { result: payload.ids });
      case _constants.RESPONSE.COUNT:
        return _extends({}, state, { total: payload.count });
      default:
        return state;
    }
  };

  return reducer;
}

function createListReducers(model) {
  if (!model.lists) {
    return {};
  }
  var listReducers = {};
  model.lists.forEach(function (list) {
    return listReducers[list.name] = listReducer(model, list);
  });
  return (0, _redux.combineReducers)(listReducers);
}