'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _normalizr = require('normalizr');

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

//TODO: Move URL related logic to APIAdapter
var ModelActions = function () {
  function ModelActions(model, config) {
    var routeParams = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var api = arguments[3];
    var requestAdapter = arguments[4];

    _classCallCheck(this, ModelActions);

    this.model = model;
    this.entitySchema = new _normalizr.schema.Entity(model.modelName);
    this.routeParams = routeParams;
    this.api = api;
    this.requestAdapter = requestAdapter;
  }

  _createClass(ModelActions, [{
    key: '_successHandler',
    value: function _successHandler(dispatch, creator) {
      return function (response) {
        console.log('request succeeded with JSON response', response);
        var actionResult = creator(response);
        var actions = _lodash2.default.isArray(actionResult) ? actionResult : [actionResult];
        console.log('firing actions:', actions);
        actions.forEach(function (action) {
          return dispatch(action);
        });
        return response;
      };
    }
  }, {
    key: '_errorHandler',
    value: function _errorHandler(dispatch) {
      var _this = this;

      return function (error) {
        console.log('request failed', error);
        var dispatchError = function dispatchError(error, message) {
          return dispatch({ type: _constants.ERROR, payload: { modelName: _this.model.modelName, error: error, message: message } });
        };
        if (error.response) {
          error.response.json().then(function (response) {
            dispatchError(response.error, error.message);
          });
        } else {
          dispatchError(error, error.message);
        }
        throw error;
        //return error
      };
    }
  }, {
    key: '_call',
    value: function _call(path, method, fetchOptions, requestCreator, successCreator) {
      var _this2 = this;

      return function (dispatch) {
        dispatch(requestCreator());
        return _this2.api.fetch(path, method, fetchOptions, _this2._successHandler(dispatch, successCreator), _this2._errorHandler(dispatch));
      };
    }
  }, {
    key: '_createAction',
    value: function _createAction(type, others) {
      return { type: type, payload: _extends({}, others, { modelName: this.model.modelName }) };
    }
  }, {
    key: '_createNormalized',
    value: function _createNormalized(type) {
      var _this3 = this;

      var singleInstance = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var listName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;

      return function (response) {
        var _this3$_createAction;

        var normalized = void 0;
        if (_lodash2.default.isArray(response)) {
          normalized = (0, _normalizr.normalize)(response, [_this3.entitySchema]);
        } else {
          normalized = (0, _normalizr.normalize)(response, _this3.entitySchema);
        }
        console.log('normalized: ', normalized);

        var actions = _lodash2.default.map(normalized.entities, function (entities, modelName) {
          return { type: _constants.RECEIVED, payload: { modelName: modelName, instances: entities } };
        });
        actions.push(_this3._createAction(type, (_this3$_createAction = {}, _defineProperty(_this3$_createAction, singleInstance ? 'id' : 'ids', normalized.result), _defineProperty(_this3$_createAction, 'listName', listName), _this3$_createAction)));
        return actions;
      };
    }
  }, {
    key: 'create',
    value: function create(data) {
      var _this4 = this;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$creat = this.requestAdapter.create(data, { apiPath: apiPath }),
          url = _requestAdapter$creat.url,
          method = _requestAdapter$creat.method,
          options = _requestAdapter$creat.options;

      return this._call(url, method, options, function () {
        return _this4._createAction(_constants.REQUEST.CREATE, { data: data });
      }, this._createNormalized(_constants.RESPONSE.CREATE, true));
    }
  }, {
    key: 'update',
    value: function update(id, data) {
      var _this5 = this;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$updat = this.requestAdapter.update(id, data, { apiPath: apiPath }),
          url = _requestAdapter$updat.url,
          method = _requestAdapter$updat.method,
          options = _requestAdapter$updat.options;

      return this._call(url, method, options, function () {
        return _this5._createAction(_constants.REQUEST.UPDATE, { id: id, data: data });
      }, this._createNormalized(_constants.RESPONSE.UPDATE, true));
    }
  }, {
    key: 'updateAll',
    value: function updateAll(where, data) {
      var _this6 = this;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$updat2 = this.requestAdapter.updateAll(where, data, { apiPath: apiPath }),
          url = _requestAdapter$updat2.url,
          method = _requestAdapter$updat2.method,
          options = _requestAdapter$updat2.options;

      return this._call(url, method, options, function () {
        return _this6._createAction(_constants.REQUEST.UPDATE_ALL, { where: where, data: data });
      }, function (response) {
        return _this6._createAction(_constants.RESPONSE.UPDATE_ALL, { count: response.count });
      });
    }
  }, {
    key: 'find',
    value: function find(filter) {
      var _this7 = this;

      var listName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : undefined;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);
      console.log('using apiPath', apiPath);

      var _requestAdapter$find = this.requestAdapter.find(filter, { apiPath: apiPath }),
          url = _requestAdapter$find.url,
          method = _requestAdapter$find.method,
          options = _requestAdapter$find.options;

      return this._call(url, method, options, function () {
        return _this7._createAction(_constants.REQUEST.FIND, { filter: filter, listName: listName });
      }, this._createNormalized(_constants.RESPONSE.FIND, false, listName));
    }
  }, {
    key: 'findById',
    value: function findById(id, filter) {
      var _this8 = this;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$findB = this.requestAdapter.findById(id, filter, { apiPath: apiPath }),
          url = _requestAdapter$findB.url,
          method = _requestAdapter$findB.method,
          options = _requestAdapter$findB.options;

      return this._call(url, method, options, function () {
        return _this8._createAction(_constants.REQUEST.FIND_BY_ID, { id: id, filter: filter });
      }, this._createNormalized(_constants.RESPONSE.FIND_BY_ID, true));
    }
  }, {
    key: 'deleteById',
    value: function deleteById(id) {
      var _this9 = this;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$delet = this.requestAdapter.deleteById(id, { apiPath: apiPath }),
          url = _requestAdapter$delet.url,
          method = _requestAdapter$delet.method,
          options = _requestAdapter$delet.options;

      return this._call(url, method, options, function () {
        return _this9._createAction(_constants.REQUEST.DELETE_BY_ID, { id: id });
      }, function (response) {
        return _this9._createAction(_constants.RESPONSE.DELETE_BY_ID, { id: id });
      });
    }
  }, {
    key: 'count',
    value: function count(where) {
      var _this10 = this;

      var listName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$count = this.requestAdapter.count(where, { apiPath: apiPath }),
          url = _requestAdapter$count.url,
          method = _requestAdapter$count.method,
          options = _requestAdapter$count.options;

      return this._call(url, method, options, function () {
        return _this10._createAction(_constants.REQUEST.COUNT, { where: where, listName: listName });
      }, function (response) {
        return _this10._createAction(_constants.RESPONSE.COUNT, { count: response.count, listName: listName });
      });
    }
  }, {
    key: 'custom',
    value: function custom(name, path, _method) {
      var _this11 = this;

      var _options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

      var apiPath = this.requestAdapter.resolveRouteParams(this.model, this.routeParams);

      var _requestAdapter$custo = this.requestAdapter.custom(name, path, _method, _options, { apiPath: apiPath }),
          url = _requestAdapter$custo.url,
          method = _requestAdapter$custo.method,
          options = _requestAdapter$custo.options;

      return this._call(url, method, options, function () {
        return _this11._createAction(_constants.REQUEST.CUSTOM, { name: name, path: path, _method: _method, _options: _options });
      }, function (response) {
        return _this11._createAction(_constants.RESPONSE.CUSTOM, { response: response, name: name });
      });
    }
  }, {
    key: 'clear',
    value: function clear() {
      return this._createAction(_constants.CLEAR, {});
    }
  }, {
    key: 'delete',
    value: function _delete(filter) {
      throw new Error('not implemented yet');
      // return this._delete(`${this.apiPath}`, {filter: JSON.stringify(filter)},
      //   () => this._createAction(REQUEST.DELETE, { filter }),
      //   (response) => _createPayload(RESPONSE.DELETE, { ids: response.ids })
      // )
    }
  }]);

  return ModelActions;
}();

exports.default = ModelActions;