'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _normalizr = require('normalizr');

var _redux = require('redux');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _constants = require('../constants');

var _listReducer = require('../list/listReducer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

exports.default = function (model) {
  var instances = function instances() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var _ref = arguments[1];
    var type = _ref.type,
        payload = _ref.payload;

    if (payload && payload.modelName !== model.modelName) {
      return state;
    }
    switch (type) {
      case _constants.RECEIVED:
        return _extends({}, state, payload.instances);
      case _constants.RESPONSE.DELETE_BY_ID:
      case _constants.RESPONSE.DELETE:
        var newState = _extends({}, state);
        return _lodash2.default.pick(newState, _lodash2.default.difference(_lodash2.default.keys(newState), [payload.id]));
      case _constants.CLEAR:
        return {};
      default:
        return state;
    }
  };

  var error = function error() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    var _ref2 = arguments[1];
    var type = _ref2.type,
        payload = _ref2.payload;

    if (payload && payload.modelName !== model.modelName) {
      return state;
    }
    switch (type) {
      case _constants.ERROR:
        return payload;
      case _constants.CLEAR:
        return null;
      default:
        return state;
    }
  };

  var DEFAULT_REQUEST = { loading: false };
  var request = function request() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_REQUEST;
    var _ref3 = arguments[1];
    var type = _ref3.type,
        payload = _ref3.payload;

    if (payload && payload.modelName !== model.modelName) {
      return state;
    }
    switch (type) {
      case _constants.REQUEST.FIND:
      case _constants.REQUEST.FIND_BY_ID:
      case _constants.REQUEST.CREATE:
      case _constants.REQUEST.UPDATE:
      case _constants.REQUEST.DELETE_BY_ID:
      case _constants.REQUEST.DELETE:
      case _constants.REQUEST.CUSTOM:
        return _extends({}, state, { loading: true });
      case _constants.RESPONSE.FIND:
      case _constants.RESPONSE.FIND_BY_ID:
      case _constants.RESPONSE.CREATE:
      case _constants.RESPONSE.UPDATE:
      case _constants.RESPONSE.DELETE_BY_ID:
      case _constants.RESPONSE.DELETE:
      case _constants.RESPONSE.CUSTOM:
      case _constants.ERROR:
        return _extends({}, state, { loading: false });
      case _constants.CLEAR:
        return _extends({}, DEFAULT_REQUEST);
      default:
        return state;
    }
  };

  var DEFAULT_LAST_STATE = { find: [], delete: [], custom: {} };
  var last = function last() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_LAST_STATE;
    var _ref4 = arguments[1];
    var type = _ref4.type,
        payload = _ref4.payload;

    if (payload && payload.modelName !== model.modelName) {
      return state;
    }

    if (payload && payload.listName) {
      return state;
    }

    switch (type) {
      case _constants.RESPONSE.FIND:
        return _extends({}, state, { find: payload.ids });
      case _constants.RESPONSE.FIND_BY_ID:
        return _extends({}, state, { findById: payload.id });
      case _constants.RESPONSE.CREATE:
        return _extends({}, state, { create: payload.id });
      case _constants.RESPONSE.UPDATE:
        return _extends({}, state, { update: payload.id });
      case _constants.RESPONSE.UPDATE_ALL:
        return _extends({}, state, { updateAll: payload.count });
      case _constants.RESPONSE.COUNT:
        return _extends({}, state, { count: payload.count });
      case _constants.RESPONSE.DELETE_BY_ID:
        //case RESPONSE.DELETE:
        var newState = _extends({}, state, { deleteById: payload.id });
        if (newState.findById == payload.id) newState.findById = null;
        if (newState.create == payload.id) newState.create = null;
        if (newState.update == payload.id) newState.update = null;
        if (newState.find.indexOf(payload.id) > -1) {
          newState.find = newState.find.filter(function (id) {
            return id !== payload.id;
          });
        }
        return newState;
      case _constants.RESPONSE.CUSTOM:
        return _extends({}, state, { custom: _extends({}, state.custom, _defineProperty({}, payload.name, payload.response)) });
      case _constants.CLEAR:
        return _extends({}, DEFAULT_LAST_STATE);
      default:
        return state;
    }
  };
  //console.log('listREducer', listReducer)
  return (0, _redux.combineReducers)({
    instances: instances,
    last: last,
    request: request,
    error: error,
    lists: (0, _listReducer.createListReducers)(model)
  });
};